@@ -217,7 +217,7 @@ async def output_fun(julius_app, logger: Logger) -> Tuple[bool, List[Optional[Tu
         julius_app.set_operation(EntityType.OUTPUT)
         if not julius_app.output_fun_exists():
             # not really know now, is derived from what has come
-            schemes_info = ([(dfs_many_fun, ("*",))], None, None)
+            schemes_info = ([(dfs_many_fun, ("*",), False)], None, None)
             julius_app._update_schemes_info(schemes_info, EntityType.OUTPUT)
         jdfs_list, collections_list = await interpret(julius_app, julius_app.collections)
         julius_app.update_metadata(collections_list)
@@ -230,15 +230,19 @@ async def output_fun(julius_app, logger: Logger) -> Tuple[bool, List[Optional[Tu
 
         collections: List[Collection] = []
         if julius_app.kafka_helper is None:
-            for jdf, (_, scheme) in zip(jdfs_list, schemes_info):
+            for jdf, (_, scheme, _) in zip(jdfs_list, schemes_info):
                 if isinstance(jdf, DF):
-                    collections.append(save_collection(jdf, julius_app.operation_id, fix_scheme=None if scheme is None or scheme[0] is None else FixScheme(schemeName=scheme[0], mode="not_check")))
+                    collection = save_collection(jdf, julius_app.operation_id, fix_scheme=None if scheme is None or scheme[0] is None else FixScheme(schemeName=scheme[0], mode="not_check"))
+                    julius_app.collection_ids.add(collection.get())
+                    collections.append(collection)
                 elif isinstance(jdf, OBJ):
                     collections.append(save_object_collection(julius_app, jdf))
                 elif isinstance(jdf, Sink):
                     raise Exception(f"Sink argument not supported for output")
                 elif isinstance(jdf, Doc) or isinstance(jdf, Docs):
-                    collections.append(save_doc(jdf, julius_app.operation_id, fix_scheme=None if scheme is None or scheme[0] is None else FixScheme(schemeName=scheme[0], mode="not_check")))
+                    collection = save_doc(jdf, julius_app.operation_id, fix_scheme=None if scheme is None or scheme[0] is None else FixScheme(schemeName=scheme[0], mode="not_check"))
+                    julius_app.collection_ids.add(collection.get(with_prefix=True))
+                    collections.append(collection)
                 else:
                     assert isinstance(jdf, DFS), "internal error: output wrong df type"
                     if scheme is None:
@@ -247,13 +251,17 @@ async def output_fun(julius_app, logger: Logger) -> Tuple[bool, List[Optional[Tu
                         assert len(jdf) == len(scheme), f"internal error: wrong schemes count {scheme}"
                     for df, subscheme in zip(jdf, scheme):
                         if isinstance(df, DF):
-                            collections.append(save_collection(df, julius_app.operation_id, fix_scheme=None if subscheme is None or subscheme[0] is None else FixScheme(schemeName=subscheme[0], mode="not_check")))
+                            collection = save_collection(df, julius_app.operation_id, fix_scheme=None if subscheme is None or subscheme[0] is None else FixScheme(schemeName=subscheme[0], mode="not_check"))
+                            julius_app.collection_ids.add(collection.get())
+                            collections.append(collection)
                         elif isinstance(df, OBJ):
                             collections.append(save_object_collection(julius_app, df))
                         elif isinstance(df, Sink):
                             raise Exception(f"Sink argument not supported for output")
                         elif isinstance(df, Doc) or isinstance(df, Docs):
-                            collections.append(save_doc(df, julius_app.operation_id, fix_scheme=None if scheme is None or scheme[0] is None else FixScheme(schemeName=scheme[0], mode="not_check")))
+                            collection = save_doc(df, julius_app.operation_id, fix_scheme=None if scheme is None or scheme[0] is None else FixScheme(schemeName=scheme[0], mode="not_check"))
+                            julius_app.collection_ids.add(collection.get(with_prefix=True))
+                            collections.append(collection)
                         else:
                             assert isinstance(df, DFS), "internal error: output wrong df type"
                             for df_i in df:
@@ -262,12 +270,16 @@ async def output_fun(julius_app, logger: Logger) -> Tuple[bool, List[Optional[Tu
                                 elif isinstance(df_i, Sink):
                                     raise Exception(f"internal error: Sink argument not supported for output")
                                 elif isinstance(df_i, Doc) or isinstance(df_i, Docs):
-                                    collections.append(save_doc(df_i, julius_app.operation_id, fix_scheme=None if scheme is None or scheme[0] is None else FixScheme(schemeName=scheme[0], mode="not_check")))
+                                    collection = save_doc(df_i, julius_app.operation_id, fix_scheme=None if scheme is None or scheme[0] is None else FixScheme(schemeName=scheme[0], mode="not_check"))
+                                    julius_app.collection_ids.add(collection.get(with_prefix=True))
+                                    collections.append(collection)
                                 else:
-                                    collections.append(save_collection(df_i, julius_app.operation_id, fix_scheme=None if subscheme is None or subscheme[0] is None else FixScheme(schemeName=subscheme[0], mode="not_check")))
+                                    collection = save_collection(df_i, julius_app.operation_id, fix_scheme=None if subscheme is None or subscheme[0] is None else FixScheme(schemeName=subscheme[0], mode="not_check"))
+                                    julius_app.collection_ids.add(collection.get())
+                                    collections.append(collection)
         elif julius_app.continue_after_processor or ok:
             for_produce_collections = []
-            for jdf, (_, scheme) in zip(jdfs_list, schemes_info):
+            for jdf, (_, scheme, _) in zip(jdfs_list, schemes_info):
                 if isinstance(jdf, DF):
                     if scheme is not None:
                         assert len(scheme) == 1, f"output wrong schemes count (should be 1): {scheme}"
