@@ -9,7 +9,7 @@ from malevich_app.docker_export.funcs import input_fun, processor_fun, output_fu
 from malevich_app.export.abstract.abstract import (Init, InputCollections, AppFunctionsInfo, FunMetadata, InitRun,
                                                    LogsResult, LogsOperation, GetAppInfo, Run, Collection, Objects,
                                                    InitPipeline, RunPipeline, RunStream, Cancel,
-                                                   PipelineStructureUpdate)
+                                                   PipelineStructureUpdate, GetState)
 from malevich_app.export.jls.JuliusPipeline import JuliusPipeline
 from malevich_app.export.jls.JuliusRegistry import JuliusRegistry
 from malevich_app.export.kafka.KafkaHelper import KafkaHelper
@@ -447,12 +447,13 @@ async def stream(run: RunStream, response: Response):
     )
 
 
-@app.post("/finish", status_code=HTTP_204_NO_CONTENT)
-async def finish(metadata: FunMetadata):    # TODO stop all running
+@app.post("/finish", status_code=HTTP_200_OK)
+async def finish(metadata: FunMetadata, response: Response):    # TODO stop all running
     state = states.get(metadata.operationId)
     if state is None:
         log_error(f"/finish wrong operationId {metadata.operationId}")
-        return Response(status_code=HTTP_404_NOT_FOUND)
+        response.status_code = HTTP_404_NOT_FOUND
+        return {}
 
     if metadata.runId is None:
         if state.pipeline is None:
@@ -461,14 +462,18 @@ async def finish(metadata: FunMetadata):    # TODO stop all running
         else:
             state.pipeline.cancel()
         states.pop(metadata.operationId, None)
-        return Response(status_code=HTTP_204_NO_CONTENT)
+        return {}
 
     if state.pipeline is None:
-        state.j_apps.pop(metadata.runId, None)      # it could fail at the init
+        j_app = state.j_apps.pop(metadata.runId, None)      # it could fail at the init
         state.pauses.pop(metadata.runId, None)
+        if j_app is not None:
+            return {"data": j_app.collection_ids}
+        else:
+            return {}
     else:
-        state.pipeline.delete_run(metadata.runId)
-    return Response(status_code=HTTP_204_NO_CONTENT)
+        collection_ids = state.pipeline.delete_run(metadata.runId)
+        return {"data": collection_ids}
 
 
 @app.post("/collection", status_code=HTTP_204_NO_CONTENT)
@@ -561,3 +566,33 @@ async def cancel(data: Cancel):
             state.pipeline.cancel(delete=False)
             state.pipeline.reset_hash(data.hash)
     return Response(status_code=HTTP_204_NO_CONTENT)
+
+
+@app.post("/state", status_code=HTTP_200_OK)
+async def get_state(data: GetState, response: Response):
+    state = states.get(data.operationId)
+    if state is None:
+        log_error(f"/state wrong operationId {data.operationId}")
+        response.status_code = HTTP_404_NOT_FOUND
+        return {}
+
+    if state.pipeline is None:
+        j_app = state.j_apps.get(data.runId)
+        if j_app is None:
+            state.logs_buffer.write(f"/state wrong runId {data.runId}\n")
+            log_error(f"/state wrong runId {data.runId} (operationId={data.operationId})")
+            response.status_code = HTTP_404_NOT_FOUND
+            return {}
+    else:
+        j_app = state.pipeline.run_japp(data.runId, data.bindId)
+        if j_app is None:
+            state.logs_buffer.write(f"/state wrong runId {data.runId} or bindId {data.bindId}\n")
+            log_error(f"/state wrong runId {data.runId} or bindId {data.bindId} (operationId={data.operationId})")
+            response.status_code = HTTP_404_NOT_FOUND
+            return {}
+
+    state_data = j_app._context.state._data
+    if data.key is not None:
+        return state_data.get(data.key)
+    else:
+        return state_data
