@@ -1,5 +1,7 @@
 import asyncio
+import inspect
 import io
+import json
 import logging
 import numpy
 import pandas as pd
@@ -12,8 +14,10 @@ import malevich_app.export.secondary.const as C
 import malevich_app.export.secondary.endpoints as end
 from malevich_app.docker_export.funcs import docker_mode
 from malevich_app.docker_export.helpers import LocalDfsSpark
-from malevich_app.export.abstract.abstract import QueryCollection, Credentials, CollectionOutside, DBQuery, TempRunScheme, \
-    InputFunctionInfo, AppFunctionsInfo, ProcessorFunctionInfo, OutputFunctionInfo, Cfg, InitInfo, ConditionFunctionInfo
+from malevich_app.export.abstract.abstract import QueryCollection, Credentials, CollectionOutside, DBQuery, \
+    TempRunScheme, \
+    InputFunctionInfo, AppFunctionsInfo, ProcessorFunctionInfo, OutputFunctionInfo, Cfg, InitInfo, \
+    ConditionFunctionInfo, TempRunSchemes
 from malevich_app.export.jls.EntityType import EntityType
 from malevich_app.export.jls.df import get_fun_info, JDF, get_fun_info_verbose, get_context_argname, get_argnames, get_context_info, \
     get_model_name, get_annotations, get_argcount
@@ -58,6 +62,7 @@ class JuliusApp:
         self.__output_cpu_bound = False
         self.__condition_cpu_bound = False
         self.__processor_is_stream = False
+        self.__processor_object_df_convert = True
         self.__input_finish_msg = None
         self.__processor_finish_msg = None
         self.__output_finish_msg = None
@@ -66,10 +71,10 @@ class JuliusApp:
         self.__processor_get_scale_part_all = None
         self.__output_drop_internal = None
         self.__condition_drop_internal = None
-        self.__schemes_info_input: Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]]]], Optional[Any], Optional[int]] = None
-        self.__schemes_info_processor: Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]]]], Optional[Any], Optional[int]] = None
-        self.__schemes_info_output: Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]]]], Optional[Any], Optional[int]] = None
-        self.__schemes_info_condition: Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]]]], Optional[Any], Optional[int]] = None
+        self.__schemes_info_input: Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]], bool]], Optional[Any], Optional[int]] = None
+        self.__schemes_info_processor: Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]], bool]], Optional[Any], Optional[int]] = None
+        self.__schemes_info_output: Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]], bool]], Optional[Any], Optional[int]] = None
+        self.__schemes_info_condition: Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]], bool]], Optional[Any], Optional[int]] = None
         self.__schemes = dict()     # defined with jls.scheme
         self.__schemes_send = set()
         self._input_id = None
@@ -114,8 +119,10 @@ class JuliusApp:
         self.continue_after_processor: bool = None
         self.__sink_names = None
         self.__cur_task = None
+        self.collection_ids = set()     # use for delete after finish
 
         self.app_cfg: Dict[str, Any] = dict()
+        self.__app_secrets: Dict[str, str] = dict()
         self.local_dfs: LocalDfs = local_dfs if local_dfs is not None else (LocalDfs() if self.docker_mode != "pyspark" else LocalDfsSpark())
         self.kafka_helper: KafkaHelper = None
         self.metadata: Dict[str, Optional[str]] = dict()
@@ -137,13 +144,15 @@ class JuliusApp:
         # logs for run_id
         self.logs_buffer: io.StringIO = None
 
-    def _init(self, operation_id: str, app_id: str, task_id: Optional[str] = None, index: Optional[int] = None, scale: Optional[int] = None, extra_collections_from: Optional[Dict[str, List[str]]] = None):
+    def _init(self, operation_id: str, app_id: str, task_id: Optional[str] = None, index: Optional[int] = None, scale: Optional[int] = None, app_secrets: Optional[Dict[str, str]] = None, extra_collections_from: Optional[Dict[str, List[str]]] = None):
         assert self.__operation_id is None, "japp already inited"
         self.__operation_id = operation_id
         self.app_id = app_id
         self.task_id = task_id
         self.__index = index
         self.__scale = scale
+        if app_secrets is not None:
+            self.__app_secrets = app_secrets
         if extra_collections_from is not None and self.__app_version == 0:
             self.__app_version = 1
             self.__override_extra_collections_names = {} if extra_collections_from is None else extra_collections_from
@@ -240,7 +249,7 @@ class JuliusApp:
         common = self._context.common if not isinstance(self._context, PreContext) else Object()
 
         if len(self.__context_funcs) == 0:
-            context = Context(self.__operation_id, self.run_id, self.app_id, self.app_cfg, self.__cfg.msg_url, self.__cfg.email, self.dag_host_port, self.dag_host_port_extra, self.secret, self.dag_host_auth, self._single_pod, self.__scale or self.state.scale, self.__index, self._login, self.logs_buffer, self._local_storage, self.__logger_fun)
+            context = Context(self.__operation_id, self.run_id, self.app_id, self.app_cfg, self.__app_secrets, self.__cfg.msg_url, self.__cfg.email, self.dag_host_port, self.dag_host_port_extra, self.secret, self.dag_host_auth, self._single_pod, self.__scale or self.state.scale, self.__index, self._login, self.logs_buffer, self._local_storage, self.__logger_fun)
             context.common = common
             self._contexts[None] = context
             self._context = context
@@ -248,7 +257,7 @@ class JuliusApp:
             context = None
             for name, context_fun in self.__context_funcs.items():
                 if context is None:
-                    context = context_fun(self.__operation_id, self.run_id, self.app_id, self.app_cfg, self.__cfg.msg_url, self.__cfg.email, self.dag_host_port, self.dag_host_port_extra, self.secret, self.dag_host_auth, self._single_pod, self.__scale or self.state.scale, self.__index, self._login, self.logs_buffer, self._local_storage, self.__logger_fun)
+                    context = context_fun(self.__operation_id, self.run_id, self.app_id, self.app_cfg, self.__app_secrets, self.__cfg.msg_url, self.__cfg.email, self.dag_host_port, self.dag_host_port_extra, self.secret, self.dag_host_auth, self._single_pod, self.__scale or self.state.scale, self.__index, self._login, self.logs_buffer, self._local_storage, self.__logger_fun)
                     context.common = common
                     context._fix_app_cfg()
                     self._contexts[name] = context
@@ -553,7 +562,7 @@ class JuliusApp:
         else:
             raise Exception("wrong operation, need drop internal")
 
-    async def get_schemes_info(self) -> Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]]]], Optional[Any], Optional[int]]:
+    async def get_schemes_info(self) -> Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]], bool]], Optional[Any], Optional[int]]:
         if self.__operation == EntityType.INPUT:
             schemes_info, ret_type, sink_index = self.__schemes_info_input
         elif self.__operation == EntityType.PROCESSOR:
@@ -566,12 +575,12 @@ class JuliusApp:
             raise Exception("wrong operation, get scheme name")
 
         res = await gather(*map(lambda x: self.__update_schemes(x[1]), schemes_info))
-        return [(x[0], res[i]) for i, x in enumerate(schemes_info)], ret_type, sink_index
+        return [(x[0], res[i], x[2]) for i, x in enumerate(schemes_info)], ret_type, sink_index
 
     def __get_scheme(self, name: str) -> Optional[str]:
         return self.__schemes.get(name, None)
 
-    async def __update_schemes_inside(self, scheme_name: Optional[str]) -> Optional[str]:
+    def __update_schemes_inside(self, scheme_name: Optional[str], schemes: List[TempRunScheme]) -> Optional[str]:
         if scheme_name is None or scheme_name == "*":
             return scheme_name
         elif isinstance(scheme_name, str):
@@ -582,9 +591,8 @@ class JuliusApp:
             scheme = self.__get_scheme(scheme_name)
             if scheme is None:  # not defined with jls.scheme
                 return scheme_name
-            temp_run_scheme = TempRunScheme(operationId=self.__operation_id, data=scheme, name=scheme_name)
             if not self.__is_local:
-                await post_request_json(end.RUN_SCHEME, temp_run_scheme.model_dump_json(), text=True, buffer=self.logs_buffer)
+                schemes.append(TempRunScheme(operationId=self.__operation_id, data=scheme, name=scheme_name))
             self.__schemes_send.add(scheme_name)
             return f"{self.__operation_id}_{scheme_name}"
         self.logs_buffer.write(f"warn: wrong scheme type: {scheme_name}\n")
@@ -594,7 +602,11 @@ class JuliusApp:
         if scheme_names is None:
             return scheme_names
         assert isinstance(scheme_names, Tuple), f"wrong scheme {scheme_names}"
-        return tuple(await gather(*map(self.__update_schemes_inside, scheme_names)))
+        schemes = []
+        res = tuple([self.__update_schemes_inside(scheme_name, schemes) for scheme_name in scheme_names])
+        if schemes:
+            await post_request_json(end.RUN_SCHEME, TempRunSchemes(data=schemes).model_dump_json(), text=True, buffer=self.logs_buffer)
+        return res
 
     def set_operation(self, operation: EntityType):
         self.__operation = operation
@@ -632,11 +644,7 @@ class JuliusApp:
                 if collections_names is not None or extra_collections_names is not None:
                     raise EntityException(f"only one way to set collections (app id={self.app_id})")
             if id not in self.__info.inputs:
-                # try:
                 input_function_info = InputFunctionInfo(id=id, name=input_fun.__name__, arguments=get_fun_info_verbose(input_fun), doc=input_fun.__doc__, collectionsNames=collections_names, extraCollectionsNames=extra_collections_names, query=query, finishMsg=finish_msg, cpuBound=cpu_bound, tags=tags, mode=mode.value)
-                # except:
-                #     self.logs_buffer.write(f"wrong input: {id}")
-                #     input_function_info = InputFunctionInfo(id=id, name=input_fun.__name__, arguments=get_fun_info_verbose(input_fun), doc=input_fun.__doc__, collectionsNames=collections_names, extraCollectionsNames=extra_collections_names, query=query, finishMsg=finish_msg, cpuBound=cpu_bound, mode=mode.value)
                 self.__info.inputs[id] = input_function_info
             else:
                 return
@@ -676,18 +684,14 @@ class JuliusApp:
         self.__input_finish_msg = finish_msg
         self.__set_collections()
 
-    def register_processor(self, processor_fun: callable, id: Optional[str], finish_msg: Optional[str], cpu_bound: bool, is_stream: bool, tags: Dict[str, str], drop_internal: bool, get_scale_part_all: bool = False):
+    def register_processor(self, processor_fun: callable, id: Optional[str], finish_msg: Optional[str], cpu_bound: bool, is_stream: bool, object_df_convert: bool, tags: Dict[str, str], drop_internal: bool, get_scale_part_all: bool = False):
         if id is None:
             id = processor_fun.__name__
         schemes_info = None
         if self.__mode == "info":
             if id not in self.__info.processors:
                 schemes_info = get_fun_info(processor_fun)
-                # try:
-                processor_function_info = ProcessorFunctionInfo(id=id, name=processor_fun.__name__, arguments=get_fun_info_verbose(processor_fun), doc=processor_fun.__doc__, finishMsg=finish_msg, cpuBound=cpu_bound, isStream=is_stream, tags=tags, contextClass=get_context_info(schemes_info[0]))
-                # except:
-                #     self.logs_buffer.write(f"wrong processor: {id}")
-                #     processor_function_info = ProcessorFunctionInfo(id=id, name=processor_fun.__name__, arguments=get_fun_info_verbose(processor_fun), doc=processor_fun.__doc__, finishMsg=finish_msg, cpuBound=cpu_bound, isStream=is_stream, contextClass=get_context_info(schemes_info[0]))
+                processor_function_info = ProcessorFunctionInfo(id=id, name=processor_fun.__name__, arguments=get_fun_info_verbose(processor_fun), doc=processor_fun.__doc__, finishMsg=finish_msg, cpuBound=cpu_bound, isStream=is_stream, objectDfConvert=object_df_convert, tags=tags, contextClass=get_context_info(schemes_info[0]))
                 self.__info.processors[id] = processor_function_info
             else:
                 return
@@ -698,6 +702,7 @@ class JuliusApp:
         self.__processor_fun = processor_fun
         self.__processor_cpu_bound = cpu_bound
         self.__processor_is_stream = is_stream
+        self.__processor_object_df_convert = object_df_convert
         if len(schemes_info[0]) > 0:
             context_fun = None
             if schemes_info[0][0][1] == CONTEXT:
@@ -726,11 +731,7 @@ class JuliusApp:
             collection_out_names = [collection_out_name]
         if self.__mode == "info":
             if id not in self.__info.outputs:
-                # try:
                 output_function_info = OutputFunctionInfo(id=id, name=output_fun.__name__, arguments=get_fun_info_verbose(output_fun), doc=output_fun.__doc__, collectionOutNames=collection_out_names, finishMsg=finish_msg, cpuBound=cpu_bound, tags=tags)
-                # except:
-                #     self.logs_buffer.write(f"wrong output: {id}")
-                #     output_function_info = OutputFunctionInfo(id=id, name=output_fun.__name__, arguments=get_fun_info_verbose(output_fun), doc=output_fun.__doc__, collectionOutNames=collection_out_names, finishMsg=finish_msg, cpuBound=cpu_bound)
                 self.__info.outputs[id] = output_function_info
             else:
                 return
@@ -770,11 +771,7 @@ class JuliusApp:
             id = condition_fun.__name__
         if self.__mode == "info":
             if id not in self.__info.conditions:
-                # try:
                 condition_function_info = ConditionFunctionInfo(id=id, name=condition_fun.__name__, arguments=get_fun_info_verbose(condition_fun), doc=condition_fun.__doc__, finishMsg=finish_msg, cpuBound=cpu_bound, tags=tags)
-                # except:
-                #     self.logs_buffer.write(f"wrong condition: {id}")
-                #     condition_function_info = ConditionFunctionInfo(id=id, name=condition_fun.__name__, arguments=get_fun_info_verbose(condition_fun), doc=condition_fun.__doc__, finishMsg=finish_msg, cpuBound=cpu_bound)
                 self.__info.conditions[id] = condition_function_info
             else:
                 return
@@ -823,11 +820,7 @@ class JuliusApp:
         if id in init_apps_update:
             enable = init_apps_update[id]
         if self.__mode == "info":
-            # try:
             init_function_info = InitInfo(id=id, enable=enable, tl=tl, prepare=prepare, argname=get_context_argname(init_fun), doc=init_fun.__doc__, cpuBound=cpu_bound, tags=tags)
-            # except:
-            #     self.logs_buffer.write(f"wrong init: {id}")
-            #     init_function_info = InitInfo(id=id, enable=enable, tl=tl, prepare=prepare, argname=get_context_argname(init_fun), doc=init_fun.__doc__, cpuBound=cpu_bound)
             self.__info.inits[id] = init_function_info
         if not enable:
             return
@@ -891,7 +884,7 @@ class JuliusApp:
             await self.__init_fail("", e, trace)
             return False
 
-    def _update_schemes_info(self, schemes_info: Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]]]], Optional[Any], bool], operation: EntityType):
+    def _update_schemes_info(self, schemes_info: Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]], bool]], Optional[Any], bool], operation: EntityType):
         if operation == EntityType.INPUT:
             self.__schemes_info_input = schemes_info
         elif operation == EntityType.PROCESSOR:
@@ -1004,6 +997,9 @@ class JuliusApp:
     def is_stream(self) -> bool:
         return self.__processor_is_stream
 
+    def object_df_convert(self) -> bool:
+        return self.__processor_object_df_convert
+
     def _cfg_dict_if_need(self, fun_id: Optional[str] = None) -> Dict[str, Any]: # maybe cfg is wrong because it is already used
         if self.__get_context_pos() == ContextPosition.NONE:
             return None
@@ -1036,6 +1032,40 @@ class JuliusApp:
             res = (False, ("processor function should return dataframes, not None", "InternalError", [], False))
         return res
 
+    def stream_wrapper(self, gen):
+        if inspect.isasyncgenfunction(gen) or inspect.isasyncgen(gen):
+            async def async_wrapper(*args, **kwargs):
+                try:
+                    with redirect_out(self.logs_buffer):
+                        async for item in gen(*args, **kwargs):
+                            if isinstance(item, str) or isinstance(item, bytes):
+                                yield item
+                            else:
+                                yield json.dumps(item)
+                except BaseException:
+                    form_exc = format_exc_skip()
+                    self.logs_buffer.write(f"{form_exc}\n")
+                    log_error(form_exc)
+                    if C.STREAM_YIELD_ERROR:
+                        yield f"error: {form_exc}"
+            return async_wrapper
+        else:
+            def wrapper(*args, **kwargs):
+                try:
+                    with redirect_out(self.logs_buffer):
+                        for item in gen(*args, **kwargs):
+                            if isinstance(item, str) or isinstance(item, bytes):
+                                yield item
+                            else:
+                                yield json.dumps(item)
+                except BaseException:
+                    form_exc = format_exc_skip()
+                    self.logs_buffer.write(f"{form_exc}\n")
+                    log_error(form_exc)
+                    if C.STREAM_YIELD_ERROR:
+                        yield f"error: {form_exc}"
+            return wrapper
+
     def cancel(self):
         if self.__cur_task is not None:
             self.__cur_task.cancel()
