@@ -7,7 +7,8 @@ from pydantic import BaseModel
 import malevich_app.export.secondary.const as C
 from malevich_app.docker_export.funcs import docker_mode
 from malevich_app.docker_export.helpers import LocalDfsSpark
-from malevich_app.docker_export.operations import get_df, get_df_share, get_df_local, get_df_object_record, get_doc_data
+from malevich_app.docker_export.operations import get_df, get_df_share, get_df_local, get_df_object_record, \
+    get_doc_data, get_df_object_convert
 from malevich_app.export.abstract.abstract import CollectionAndScheme, FixScheme, Cfg
 from malevich_app.export.abstract.pipeline import PullCollectionPolicy
 from malevich_app.export.jls.EntityType import EntityType
@@ -19,6 +20,7 @@ from malevich_app.export.secondary.EntityException import LOAD_COLLECTION_FAIL_M
 from malevich_app.export.secondary.collection.Collection import Collection
 from malevich_app.export.secondary.collection.CompositeCollection import CompositeCollection
 from malevich_app.export.secondary.collection.ComputeCollection import ComputeCollection
+from malevich_app.export.secondary.collection.DummyCollection import DummyCollection
 from malevich_app.export.secondary.collection.FileCollection import FileCollection
 from malevich_app.export.secondary.collection.JsonCollection import JsonCollection
 from malevich_app.export.secondary.collection.LocalCollection import LocalCollection
@@ -73,7 +75,7 @@ class JuliusCollectionHolder:   # TODO gc
             res.append(scheme_name)
         return tuple(res)
 
-    async def __update_collection(self, id: int, collection_id: str, scheme: Optional[str], force_reload: bool = False):
+    async def __update_collection(self, id: int, run_id: str, collection_id: str, scheme: Optional[str], force_reload: bool = False):
         if not force_reload:    # FIXME twice in one app
             internal_id = self.__internal_id(collection_id, scheme)
             if self.__internal_id_to_local_df_id[internal_id] != -1:
@@ -86,7 +88,7 @@ class JuliusCollectionHolder:   # TODO gc
             res = {"result": "" if new_id is None else new_id}
         else:
             fix_scheme = None if scheme is None else FixScheme(schemeName=scheme)
-            collection_and_scheme = CollectionAndScheme(operationId=self.__operation_id, collectionId=collection_id, hostedAppId=C.APP_ID, secret=self.__secret, fixScheme=fix_scheme)
+            collection_and_scheme = CollectionAndScheme(operationId=self.__operation_id, runId=run_id, collectionId=collection_id, hostedAppId=C.APP_ID, secret=self.__secret, fixScheme=fix_scheme)
             res = await post_request_json(MONGO_LOAD(self.__dag_host_port), collection_and_scheme.model_dump_json(), buffer=self.__logs_buffer, fail_info=f"{LOAD_COLLECTION_FAIL_MSG}: {collection_id}", operation_id=self.__operation_id, auth_header=self.__dag_host_auth)
         if collection_id.startswith(JsonCollection.prefix) or is_docs_scheme(scheme):
             df_or_doc, _, metadata = get_collection_json(collection_id, self.__operation_id)
@@ -111,7 +113,7 @@ class JuliusCollectionHolder:   # TODO gc
             self.__alt_collections_by_bind_id_and_iter[run_id] = alt_collections_by_bind_id
         return alt_collections_by_bind_id
 
-    def add_collection_by_id(self, collection_id: str, scheme: Optional[str], on_start: bool = True) -> Collection:
+    def add_collection_by_id(self, run_id: str, collection_id: str, scheme: Optional[str], on_start: bool = True) -> Collection:
         internal_id = self.__internal_id(collection_id, scheme)
         if collection_id.startswith(ObjectCollection.prefix):
             coll = ObjectCollection(collection_id)
@@ -132,25 +134,27 @@ class JuliusCollectionHolder:   # TODO gc
         if on_start:
             if self.__pull_policy == PullCollectionPolicy.FORCE_RELOAD_ALL:
                 if id not in self.__to_load:
-                    self.__to_load[id] = self.__update_collection(id, collection_id, scheme, True)
+                    self.__to_load[id] = self.__update_collection(id, run_id, collection_id, scheme, True)
             else:
                 if id not in self.__to_load_temp:
-                    self.__to_load_temp[id] = self.__update_collection(id, collection_id, scheme)
+                    self.__to_load_temp[id] = self.__update_collection(id, run_id, collection_id, scheme)
         else:
             if id not in self.__to_load_temp:
                 force_reload = self.__pull_policy == PullCollectionPolicy.FORCE_RELOAD_ALL or self.__pull_policy == PullCollectionPolicy.FORCE_RELOAD
-                self.__to_load_temp[id] = self.__update_collection(id, collection_id, scheme, force_reload)
+                self.__to_load_temp[id] = self.__update_collection(id, run_id, collection_id, scheme, force_reload)
 
         return coll
 
-    def add_collection_by_name(self, collection_name: str, scheme: Optional[str]) -> Collection:
+    def add_collection_by_name(self, run_id: str, collection_name: str, scheme: Optional[str], is_optional: bool) -> Collection:
         async def add(collections: Dict[str, str]) -> Collection:
             collection_id = collections.get(collection_name)
             if collection_id is None:
-                raise EntityException(f"collection with name={collection_name} not exist in config collections")
-            return self.add_collection_by_id(collection_id, scheme, False)
+                if not is_optional:
+                    raise EntityException(f"collection with name={collection_name} not exist in config collections")
+                return DummyCollection()
+            return self.add_collection_by_id(run_id, collection_id, scheme, False)
 
-        return ComputeCollection(add)
+        return ComputeCollection(add, is_optional)
 
     def add_collection_by_result(self, run_id: str, bind_processor_id_from: str, iteration: int, *, key: Optional[Tuple[str, str, int, Optional[int]]], coll: PromisedCollection) -> Collection:
         collections_by_bind_id_and_iter = self.__collections_by_bind_id_struct(run_id)
@@ -263,6 +267,10 @@ class JuliusCollectionHolder:   # TODO gc
             elif isinstance(collection, ObjectCollection):
                 if scheme == OBJ.__name__ or (julius_app.get_operation() == EntityType.OUTPUT and not julius_app.output_fun_exists() and scheme is None):
                     df, coll = await get_df_object(julius_app, collection.get())
+                elif julius_app.object_df_convert():
+                    if scheme is not None:
+                        raise EntityException(f"wrong conversation: DF from OBJ, scheme should be None or Any, found {scheme} (id={julius_app._processor_id})")
+                    df, coll = get_df_object_convert(julius_app, collection.get())
                 else:
                     if scheme is not None and scheme != "obj":
                         raise EntityException(f"wrong conversation: DF from OBJ, scheme should be None, Any or \"obj\", found {scheme} (id={julius_app._processor_id})")
@@ -302,12 +310,14 @@ class JuliusCollectionHolder:   # TODO gc
                 return VariantList(res)
             elif isinstance(collection, JsonCollection):
                 df, coll = await get_doc_data(julius_app, collection.get(with_prefix=True), None if scheme is None else FixScheme(schemeName=scheme, mode=collection.get_mode()))
+            elif isinstance(collection, DummyCollection):
+                df, coll = collection.get(), collection
             else:
                 raise Exception("wrong Collection type")
 
-            if df is None:
+            if df is None and not isinstance(coll, DummyCollection):
                 raise EntityException(f"internal error: df is None. collection type = {type(coll)}; scheme = {scheme} (id={julius_app._processor_id})")
-            if julius_app.need_drop_internal and not isinstance(df, str) and not isinstance(df, Dict) and not isinstance(df, List) and not issubclass(df.__class__, BaseModel):  # for OBJ and Doc and Docs
+            if julius_app.need_drop_internal and df is not None and not isinstance(df, str) and not isinstance(df, Dict) and not isinstance(df, List) and not issubclass(df.__class__, BaseModel):  # for OBJ and Doc and Docs
                 df.drop("__id__", axis=1, inplace=True, errors='ignore')
                 df.drop("__name__", axis=1, inplace=True, errors='ignore')
             if isinstance(df, List) and isinstance(coll, LocalCollection) and coll.is_doc():
