@@ -7,12 +7,11 @@ from copy import deepcopy
 from functools import cached_property
 from typing import Dict, Any, Optional, Tuple, List, Callable, Set
 import pandas as pd
-from asyncio import gather
 from mypy_extensions import VarArg
 from pydantic import BaseModel
 from malevich_app.export.abstract.abstract import PipelineApp, Cfg, AppFunctionsInfo, InputFunctionInfo, \
     ProcessorFunctionInfo, \
-    OutputFunctionInfo, ConditionFunctionInfo, InitInfo, AnyInit, TempRunScheme, FunctionInfo
+    OutputFunctionInfo, ConditionFunctionInfo, InitInfo, AnyInit, TempRunScheme, FunctionInfo, TempRunSchemes
 from malevich_app.export.jls.EntityType import EntityType
 from malevich_app.export.jls.JuliusApp import JuliusApp
 from malevich_app.export.jls.LocalLogsBuffer import LocalLogsBuffer
@@ -35,8 +34,9 @@ class JuliusEmptyRegistry:
     def __init__(self):
         pass
 
-    def create_app(self, init: AnyInit, bind_id: str, cfg: Cfg, pipeline_app: Optional[PipelineApp] = None, app_cfg_extension: Optional[str] = None) -> JuliusApp:
-        raise Exception("create app i empty registry")
+    def create_app(self, init: AnyInit, bind_id: str, cfg: Optional[Cfg], pipeline_app: Optional[PipelineApp] = None, app_cfg: Any = None, app_secrets: Optional[Dict[str, str]] = None,
+                   app_cfg_extension: Optional[str] = None, local_dfs: Optional[LocalDfs] = None, *args, **kwargs) -> JuliusApp:
+        raise Exception("create app in empty registry")
 
     def register_input(self, fun: callable, id: Optional[str], *args, **kwargs):
         pass
@@ -58,6 +58,8 @@ class JuliusEmptyRegistry:
 
 
 class JuliusRegistry(JuliusEmptyRegistry):
+    _libs = ["malevich", "malevich.square", "malevich_app.square"]
+
     def __init__(self, import_dirs: Optional[List[str]] = None, logger_fun: Optional[Callable[[str, Optional[str], Optional[str], bool], logging.Logger]] = None, with_import_restrictions: bool = True):
         super().__init__()
         Wrapper.app = self
@@ -75,12 +77,13 @@ class JuliusRegistry(JuliusEmptyRegistry):
         self.__schemes: Dict[str, Any] = {}
         self.__inits: Dict[str, Any] = {}
 
-        self.__schemes_info_processors: Dict[str, Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]]]], Optional[Any], Optional[int]]] = {}
-        self.__schemes_info_outputs: Dict[str, Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]]]], Optional[Any], Optional[int]]] = {}
-        self.__schemes_info_conditions: Dict[str, Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]]]], Optional[Any], Optional[int]]] = {}
+        self.__schemes_info_processors: Dict[str, Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]], bool]], Optional[Any], Optional[int]]] = {}
+        self.__schemes_info_outputs: Dict[str, Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]], bool]], Optional[Any], Optional[int]]] = {}
+        self.__schemes_info_conditions: Dict[str, Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]], bool]], Optional[Any], Optional[int]]] = {}
         self.__base_j_apps: Dict[str, Dict[str, JuliusApp]] = {}        # operation_id -> bind_id -> base_j_app
 
         self.__exist_schemes = set()
+        self.__imported_schemes = set()
         self.__register()
         Wrapper.app = JuliusEmptyRegistry()
 
@@ -90,11 +93,10 @@ class JuliusRegistry(JuliusEmptyRegistry):
         self.__is_local = True
 
     def __register(self):
-        lib_modules = [importlib.import_module("malevich"), importlib.import_module("malevich.square"), importlib.import_module("malevich_app.square")] # TODO import "defaults"
+        lib_modules = {lib: importlib.import_module(lib) for lib in self._libs}
 
         for dir in self.__import_dirs:
             assert os.path.isdir(dir), f"{dir} directory should exist"
-        imported = set()
 
         global _registered
         for import_dir in self.__import_dirs:
@@ -114,19 +116,25 @@ class JuliusRegistry(JuliusEmptyRegistry):
                                     importlib.reload(module)
                                 reimport = False
                             except ImportError as er:
-                                if er.msg.startswith("cannot import name '") and ("malevich" in er.name or "square" in er.name) and not C.REGISTER_RAISE_ON_ERROR:
+                                if er.msg.startswith("cannot import name '") and not C.REGISTER_RAISE_ON_ERROR:
+                                    lib_module = None
+                                    for lib, module in lib_modules.items():
+                                        if lib == er.name:
+                                            lib_module = module
+                                            break
+                                    if lib_module is None:
+                                        raise er
+
                                     failed = True
                                     msg = er.msg.replace("cannot import name '", "")
                                     scheme_name = msg[:msg.find("'")]
-                                    if scheme_name in imported:
+                                    if scheme_name in self.__imported_schemes:
                                         self.logs_buffer.write(f"error: scheme \"{scheme_name}\" not found, reimport fail\n")
                                         reimport = False
                                     else:
-                                        # self.logs_buffer.write(f"warning: scheme \"{scheme_name}\" not found\n")
-                                        imported.add(scheme_name)
+                                        self.__imported_schemes.add(scheme_name)
                                         update_dummy_scheme(scheme_name)
-                                        for lib_module in lib_modules:  # TODO ?
-                                            importlib.reload(lib_module)
+                                        importlib.reload(lib_module)
                                 else:
                                     raise er
         _registered = True
@@ -141,11 +149,14 @@ class JuliusRegistry(JuliusEmptyRegistry):
             fun_info.tags = None
         return fun_info
 
+    def imported_schemes(self) -> Set[str]:
+        return self.__imported_schemes
+
     @cached_property
     def info(self) -> AppFunctionsInfo:
         return AppFunctionsInfo(
-            inputs={id: self.__function_info_with_tags(InputFunctionInfo(id=id, name=args[0].__name__, arguments=get_fun_info_verbose(args[0]), doc=args[0].__doc__, finishMsg=kwargs["finish_msg"], mode=kwargs["mode"].value, cpuBound=kwargs["cpu_bound"]), tags=kwargs["tags"]) for id, (args, kwargs) in self.__inputs.items()},
-            processors={id: self.__function_info_with_tags(ProcessorFunctionInfo(id=id, name=args[0].__name__, arguments=get_fun_info_verbose(args[0]), doc=args[0].__doc__, finishMsg=kwargs["finish_msg"], contextClass=get_context_info(get_fun_info(args[0])[0]), cpuBound=kwargs["cpu_bound"], isStream=kwargs["is_stream"]), tags=kwargs["tags"]) for id, (args, kwargs) in self.__processors.items()},     # TODO contextClass
+            inputs={id: self.__function_info_with_tags(InputFunctionInfo(id=id, name=args[0].__name__, arguments=get_fun_info_verbose(args[0]), doc=args[0].__doc__, finishMsg=kwargs["finish_msg"], mode=kwargs["mode"].value, cpuBound=kwargs["cpu_bound"]), tags=kwargs["tags"]) for id, (args, kwargs) in self.__inputs.items()},   # object_df_convert: bool
+            processors={id: self.__function_info_with_tags(ProcessorFunctionInfo(id=id, name=args[0].__name__, arguments=get_fun_info_verbose(args[0]), doc=args[0].__doc__, finishMsg=kwargs["finish_msg"], contextClass=get_context_info(get_fun_info(args[0])[0]), cpuBound=kwargs["cpu_bound"], isStream=kwargs["is_stream"], objectDfConvert=kwargs["object_df_convert"]), tags=kwargs["tags"]) for id, (args, kwargs) in self.__processors.items()},     # TODO contextClass
             outputs={id: self.__function_info_with_tags(OutputFunctionInfo(id=id, name=args[0].__name__, arguments=get_fun_info_verbose(args[0]), doc=args[0].__doc__, finishMsg=kwargs["finish_msg"], collectionOutNames=kwargs["collection_out_names"] if (collection_out_name := kwargs["collection_out_name"]) is None else [collection_out_name], cpuBound=kwargs["cpu_bound"]), tags=kwargs["tags"]) for id, (args, kwargs) in self.__outputs.items()},
             conditions={id: self.__function_info_with_tags(ConditionFunctionInfo(id=id, name=args[0].__name__, arguments=get_fun_info_verbose(args[0]), doc=args[0].__doc__, finishMsg=kwargs["finish_msg"], cpuBound=kwargs["cpu_bound"]), tags=kwargs["tags"]) for id, (args, kwargs) in self.__conditions.items()},
             schemes={id: json_schema(cl) for id, cl in self.__schemes.items()},
@@ -168,20 +179,19 @@ class JuliusRegistry(JuliusEmptyRegistry):
             await self.update_schemes_pipeline(operation_id, prefix=False)
 
     async def update_schemes_pipeline(self, operation_id: str, prefix: bool = True):
-        tasks = []
+        schemes = []
         for name, cl in self.__schemes.items():
             data = json_schema(cl)
-            temp_run_scheme = TempRunScheme(operationId=operation_id, data=data, name=name, prefix=prefix)
             if not self.__is_local or C.WS is not None:
-                tasks.append(post_request_json(end.RUN_SCHEME, temp_run_scheme.model_dump_json(), text=True, buffer=self.logs_buffer))
-        if len(tasks) > 0:
-            await gather(*tasks)
+                schemes.append(TempRunScheme(operationId=operation_id, data=data, name=name, prefix=prefix))
+        if schemes:
+            await post_request_json(end.RUN_SCHEME, TempRunSchemes(data=schemes).model_dump_json(), text=True, buffer=self.logs_buffer)
 
     @property
     def internal_schemes(self) -> Set[str]:
         return set(self.__schemes.keys())
 
-    def get_schemes_info(self, id: str, operation: EntityType) -> Optional[Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]]]], Optional[Any], Optional[int]]]:
+    def get_schemes_info(self, id: str, operation: EntityType) -> Optional[Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]], bool]], Optional[Any], Optional[int]]]:
         if operation == EntityType.PROCESSOR:
             res = self.__schemes_info_processors.get(id)
         elif operation == EntityType.OUTPUT:
@@ -194,7 +204,7 @@ class JuliusRegistry(JuliusEmptyRegistry):
             raise Exception(f"{operation.value} with id={id} not found")
         return res
 
-    def create_app(self, init: AnyInit, bind_id: str, cfg: Optional[Cfg], pipeline_app: Optional[PipelineApp] = None, app_cfg: Any = None, app_cfg_extension: Optional[str] = None, local_dfs: Optional[LocalDfs] = None, *args, **kwargs) -> JuliusApp:
+    def create_app(self, init: AnyInit, bind_id: str, cfg: Optional[Cfg], pipeline_app: Optional[PipelineApp] = None, app_cfg: Any = None, app_secrets: Optional[Dict[str, str]] = None, app_cfg_extension: Optional[str] = None, local_dfs: Optional[LocalDfs] = None, *args, **kwargs) -> JuliusApp:
         operation_id = init.operationId
         is_init = pipeline_app is not None
 
@@ -203,7 +213,6 @@ class JuliusRegistry(JuliusEmptyRegistry):
                 logs_buffer = io.StringIO()
             else:
                 logs_buffer = LocalLogsBuffer(self.__logger_fun, operation_id, None, bind_id)
-            assert app_cfg_extension is None, "override cfg should be None in init"
             local_storage = self.__local_storage if C.WS is None else None
             j_app = JuliusApp(version=2, local_dfs=local_dfs, exist_schemes=self.__exist_schemes, local_storage=local_storage, logger_fun=self.__logger_fun)
             op_base_j_apps = self.__base_j_apps.get(operation_id)
@@ -214,8 +223,7 @@ class JuliusRegistry(JuliusEmptyRegistry):
 
             j_app.prepare(proc_id=pipeline_app.processorId, out_id=pipeline_app.outputId, cond_id=pipeline_app.conditionId)
             j_app.set_app_mode("init")
-            j_app._init(operation_id, bind_id, *args, **kwargs)
-
+            j_app._init(operation_id, bind_id, *args, **kwargs, app_secrets=app_secrets)
             j_app.debug_mode = init.debugMode
             j_app.dag_host_port = C.DAG_HOST_PORT(init.dagHost)
             j_app.dag_host_port_extra = init.dagUrlExtra
@@ -233,12 +241,6 @@ class JuliusRegistry(JuliusEmptyRegistry):
             j_app = self.__base_j_apps.get(operation_id, {}).get(bind_id)
             if self.__logger_fun is not None:
                 j_app.logs_buffer.update_run_id(init.runId)
-            if app_cfg_extension is not None:  # TODO fix it if not copy later
-                try:
-                    app_cfg_extension = json.loads(app_cfg_extension)
-                except BaseException as ex:
-                    j_app.logs_buffer.write(f"extra app config load fail: {ex}\n")
-                    app_cfg_extension = {}
 
             assert j_app is not None, f"base julius app for operation_id={operation_id} and bind_id={bind_id} not exist"
             j_app.set_app_mode("run")
@@ -256,7 +258,13 @@ class JuliusRegistry(JuliusEmptyRegistry):
                 j_app.set_cfg(cfg)
             else:
                 assert j_app.exist_cfg(), "config should set"
-            j_app.set_context()
+
+        if app_cfg_extension is not None:
+            try:
+                app_cfg_extension = json.loads(app_cfg_extension)
+            except BaseException as ex:
+                j_app.logs_buffer.write(f"extra app config load fail: {ex}\n")
+                app_cfg_extension = {}
 
         if is_init:
             if pipeline_app.processorId is not None:
@@ -282,6 +290,8 @@ class JuliusRegistry(JuliusEmptyRegistry):
             for init_args_kwargs in self.__inits.values():
                 args, kwargs = init_args_kwargs
                 j_app.register_init(*args, **kwargs)
+            if app_cfg_extension is not None:
+                j_app.app_cfg.update(app_cfg_extension)
             j_app.set_context()
             return j_app
 
@@ -289,7 +299,8 @@ class JuliusRegistry(JuliusEmptyRegistry):
         local_dfs_temp, j_app.local_dfs = j_app.local_dfs, None
         copy_j_app = deepcopy(j_app)
         if app_cfg_extension is not None:
-            copy_j_app.app_cfg.update(app_cfg_extension)    # TODO check updated context.app_cfg
+            copy_j_app.app_cfg.update(app_cfg_extension)
+        copy_j_app.set_context()
         copy_j_app._context.common, j_app._context.common = common, common
         copy_j_app.local_dfs, j_app.local_dfs = local_dfs_temp, local_dfs_temp
         return copy_j_app
