@@ -3,7 +3,7 @@ import json
 import os
 from functools import cached_property, cache
 import pandas as pd
-from typing import Generic, TypeVar, List, Any, Optional, Union, Tuple, Callable, ForwardRef, Dict, Iterator
+from typing import Generic, TypeVar, List, Any, Optional, Union, Tuple, Callable, ForwardRef, Dict, Iterator, get_args, get_origin
 from pydantic import BaseModel
 from pydantic.v1.json import pydantic_encoder
 from typing_extensions import TypeVarTuple, Unpack
@@ -20,6 +20,15 @@ def __is_tuple(type):
     return str(type).startswith("typing.Tuple[") or str(type).startswith("tuple[")
 
 
+def __is_optional(type_) -> bool:
+    return get_origin(type_) is Union and type(None) in get_args(type_)
+
+
+def __unwrap_optional(type_):    # __is_optional should be True
+    args = tuple(a for a in get_args(type_) if a is not type(None))
+    return args[0] if args else None
+
+
 def _is_DF(type) -> bool:
     return hasattr(type, "__origin__") and type.__origin__.__module__ == __name__ and type.__origin__.__name__ == "DF"
 
@@ -513,10 +522,12 @@ JDF = Union[DF, DFS, Sink, OBJ, Doc, Docs]
 dfs_many_fun = lambda *dfs: DFS[M[Any]]().init(*dfs, nested=False)
 
 
-def get_fun_info(fun: callable, *, by_names: bool = False) -> Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]]]], Optional[Any], Optional[int]]:     # TODO(not ignore return later)
+def get_fun_info(fun: callable, *, by_names: bool = False) -> Tuple[List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]], bool]], Optional[Any], Optional[int]]:     # TODO(not ignore return later)
     def __name(type: Any, repr: bool = False) -> Optional[str]:
         if type is None or type is Any:
             return "" if repr else None
+        if __is_optional(type):
+            return __name(__unwrap_optional(type), repr)
         if isinstance(type, str):
             return type
         if isinstance(type, ForwardRef):
@@ -534,9 +545,15 @@ def get_fun_info(fun: callable, *, by_names: bool = False) -> Tuple[List[Tuple[C
             return "" if repr else None
         return type.__name__
 
-    def __inside_types(type) -> Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]]]:      # TODO check nested types not in this module (for example - DF[DF])
+    def __inside_types_full(type) -> Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]], bool]:
+        f, name, is_optional = __inside_types(type)
+        if is_optional:
+            return lambda x: f(x) if x is not None else None, name, is_optional
+        return f, name, is_optional
+
+    def __inside_types(type, is_optional: bool = False) -> Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]], bool]:      # TODO check nested types not in this module (for example - DF[DF])
         if _is_DF(type):
-            return type, (__name(type(None)._scheme_cls),)
+            return type, (__name(type(None)._scheme_cls),), is_optional
         elif _is_DFS(type):
             temp = type()
             sz = len(temp.__orig_class__.__args__)
@@ -561,7 +578,7 @@ def get_fun_info(fun: callable, *, by_names: bool = False) -> Tuple[List[Tuple[C
                     schemes.append(f"{DOC_SCHEME_PREFIX}{__name(df._scheme_cls, repr=True)}")
                 else:
                     assert False, f"internal error: unknown DFS type: {type(df)}"
-            return lambda *dfs: type().init(*dfs, nested=False), tuple(schemes)
+            return lambda *dfs: type().init(*dfs, nested=False), tuple(schemes), is_optional
         elif __is_Sink(type):
             temp = type()
             types = temp.__orig_class__.__args__
@@ -596,51 +613,53 @@ def get_fun_info(fun: callable, *, by_names: bool = False) -> Tuple[List[Tuple[C
                 schemes.append(f"{DOC_SCHEME_PREFIX}{__name(data._scheme_cls, repr=True)}")
             else:
                 assert False, f"internal error: unknown Sink type: {type(data)}"
-            return lambda *list_dfs: type().init(*list_dfs), tuple(schemes)
+            return lambda *list_dfs: type().init(*list_dfs), tuple(schemes), is_optional
         elif __is_OBJ(type):
-            return type, ("OBJ",)
+            return type, ("OBJ",), is_optional
         elif __is_Context(type):
-            return type, (CONTEXT_TYPE,)
+            return type, (CONTEXT_TYPE,), is_optional
         elif _is_Doc(type):
-            return lambda doc: type(doc).init(), (f"{DOC_SCHEME_PREFIX}{__name(type(None)._scheme_cls, repr=True)}",)
+            return lambda doc: type(doc).init(), (f"{DOC_SCHEME_PREFIX}{__name(type(None)._scheme_cls, repr=True)}",), is_optional
         elif _is_Docs(type):
-            return lambda docs: type(docs).init(), (f"{DOCS_SCHEME_PREFIX}{__name(type(None)._scheme_cls, repr=True)}",)
+            return lambda docs: type(docs).init(), (f"{DOCS_SCHEME_PREFIX}{__name(type(None)._scheme_cls, repr=True)}",), is_optional
         elif __is_tuple(type):
             typef = lambda *dfs: DFS[type.__args__]().init(*dfs, nested=False)
-            return typef, tuple(map(__name, type.__args__))
+            return typef, tuple(map(__name, type.__args__)), is_optional
         elif isinstance(type, List):
             typef = lambda *dfs: DFS[type]().init(*dfs, nested=False)
-            return typef, tuple(map(__name, type))
+            return typef, tuple(map(__name, type)), is_optional
+        elif __is_optional(type):
+            return __inside_types(__unwrap_optional(type), is_optional=True)
         else:
             name = __name(type)
             if name == CONTEXT_TYPE:
-                return None, (name,)
+                return None, (name,), False
             if name is None:
                 typef = lambda *dfs: ((Doc[type](*dfs).init() if isinstance(dfs[0], Dict) or issubclass(dfs[0].__class__, BaseModel) else
                                       Docs[type](*dfs).init() if isinstance(dfs[0], List) else DF[type](*dfs))) \
                     if len(dfs) == 1 else DFS[M[type]]().init(*dfs, nested=False)
-                return typef, None
+                return typef, None, is_optional
             if name == "DF" and hasattr(type, "__module__") and type.__module__ == __name__:
-                return __inside_types(DF[Any])
+                return __inside_types(DF[Any], is_optional)
             if name == "DFS" and hasattr(type, "__module__") and type.__module__ == __name__:
-                return __inside_types(DFS[M[Any]])
+                return __inside_types(DFS[M[Any]], is_optional)
             if name == "Sink" and hasattr(type, "__module__") and type.__module__ == __name__:
-                return __inside_types(Sink[Any])
+                return __inside_types(Sink[Any], is_optional)
             if name.startswith(DOC_SCHEME_PREFIX) and hasattr(type, "__module__") and type.__module__ == __name__:
-                return __inside_types(Doc[Any])
+                return __inside_types(Doc[Any], is_optional)
             if name.startswith(DOCS_SCHEME_PREFIX) and hasattr(type, "__module__") and type.__module__ == __name__:
-                return __inside_types(Docs[Any])
+                return __inside_types(Docs[Any], is_optional)
             typef = DF[type]
-            return typef, (name,)
+            return typef, (name,), is_optional
 
     types_dict = get_annotations(fun)
-    ret_type = None if "return" not in types_dict else __inside_types(types_dict.pop("return"))[1]
+    ret_type = None if "return" not in types_dict else __inside_types(types_dict.pop("return"))[1]  # ignore is Optional flag
     if by_names:
         types = {}
         sink_index = None
         for i, varname in enumerate(get_argnames(fun)):
             varname_type = types_dict.get(varname, None)
-            types[varname] = __inside_types(varname_type)
+            types[varname] = __inside_types_full(varname_type)
 
             if __is_Sink_full(varname_type):
                 assert sink_index is None, "double Sink"
@@ -650,7 +669,7 @@ def get_fun_info(fun: callable, *, by_names: bool = False) -> Tuple[List[Tuple[C
         sink_index = None
         for i, varname in enumerate(get_argnames(fun)):
             varname_type = types_dict.get(varname, None)
-            types.append(__inside_types(varname_type))
+            types.append(__inside_types_full(varname_type))
 
             if __is_Sink_full(varname_type):
                 assert sink_index is None, "double Sink"
@@ -673,6 +692,8 @@ def get_fun_info_verbose(fun: callable) -> List[Tuple[str, Optional[str]]]:
     def __name(type: Any, none_value: Optional[str] = "Any") -> Optional[str]:
         if type is None or type is Any:
             return none_value
+        if __is_optional(type):
+            return __name(__unwrap_optional(type), none_value)
         if isinstance(type, str):
             return f'"{type}"'
         if isinstance(type, ForwardRef):
@@ -743,6 +764,9 @@ def get_fun_info_verbose(fun: callable) -> List[Tuple[str, Optional[str]]]:
             return f"DFS[{', '.join(map(__name, type.__args__))}]"
         elif isinstance(type, List):
             return f"DFS[{', '.join(map(__name, type))}]"
+        elif __is_optional(type):
+            inside = __inside_types(__unwrap_optional(type))
+            return f"Optional[{inside}]"
         else:
             name = __name(type, none_value=None)
             if name == CONTEXT_TYPE:
@@ -783,7 +807,7 @@ def get_context_argname(fun: callable) -> Optional[str]:    # only for init
     raise Exception(f"too many arguments in init: {count}")
 
 
-def get_context_info(schemes_info: List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]]]]) -> Optional[Dict[str, Any]]:
+def get_context_info(schemes_info: List[Tuple[Callable[[VarArg(pd.DataFrame)], JDF], Optional[Tuple[Optional[str], ...]], bool]]) -> Optional[Dict[str, Any]]:
     context_cl = None
     if len(schemes_info) > 0:
         if schemes_info[0][1] == CONTEXT:
@@ -848,7 +872,7 @@ def get_context_info(schemes_info: List[Tuple[Callable[[VarArg(pd.DataFrame)], J
 #     b: int
 #
 #
-# def fun3(df: OBJ, dfs: DFS['a', OBJ], ctx: Context[A]):
+# def fun3(df: OBJ, dfs: Optional[DFS['a', OBJ]], ctx: Context[A]):
 #     pass
 #
 #
