@@ -1,11 +1,11 @@
 from asyncio import Future
 from io import StringIO
-from typing import Optional, List, Dict, Any
+from typing import Optional, List, Dict, Any, Union, Set
 from malevich_app.export.abstract.abstract import App
 
 
 class State:
-    def __init__(self, operation_id: str, schemes_names: List[str], scale: Optional[int] = None, *, app: Optional[App] = None, is_init_app: Optional[bool] = None, logs_buffer: Optional[StringIO] = None):
+    def __init__(self, operation_id: str, schemes_names: Union[List[str], Set[str]], scale: Optional[int] = None, *, app: Optional[App] = None, is_init_app: Optional[bool] = None, logs_buffer: Optional[StringIO] = None):
         self.operation_id = operation_id
         self.schemes_names = set(schemes_names)
         self.scale = scale
