@@ -17,8 +17,11 @@ class PauseModel(Generic[T]):
         self.__pauses[id] = fut
         data = await fut
 
-        if self.__model is not None and issubclass(self.__model, BaseModel):
-            return self.__model.model_validate_json(data)
+        if self.__model is not None:
+            if issubclass(self.__model, BaseModel):
+                return self.__model.model_validate_json(data)
+            elif self.__model == str:
+                return data.decode("utf-8")
         return data
 
 
