@@ -29,6 +29,7 @@ from malevich_app.export.abstract.abstract import Keys, KeysValues, Info, Messag
 from malevich_app.export.jls.LocalLogsBuffer import LocalLogsBuffer
 from malevich_app.export.jls.df import OBJ, JDF, DFS, Sink, Docs, DF
 from malevich_app.export.request.dag_requests import send_post_dag, send_delete_dag
+from malevich_app.jls_lib.journal import CollectBuffer, StateProxy, JournalProxy
 from malevich_app.export.secondary.LogHelper import log_info, log_error, log_warn
 from malevich_app.export.secondary.endpoints import GET_KEYS_VALUES_RAW, GET_KEYS_VALUES, GET_KEYS_VALUES_ALL, POST_KEYS_VALUES_RAW, \
     POST_KEYS_VALUES, DELETE_KEYS_VALUES_ALL, GET_OBJ_STORAGE, GET_OBJ_STORAGE_ALL, POST_OBJ_STORAGE, \
@@ -463,13 +464,14 @@ class Context(Generic[MinimalCfg]):
                                   auth_header=self.__dag_host_auth)
 
     """context for run"""
-    def __init__(self, operation_id: str = None, run_id: str = None, app_id: str = None, app_cfg: Dict[str, Any] = None, msg_url: str = None, email: Optional[str] = None, dag_host_port: str = None, dag_host_port_extra: str = None, secret: str = None, dag_host_auth: str = None, single_pod: bool = None, indexes: int = None, index: Optional[int] = None, login: str = None, japp_logs_buffer: io.StringIO = None, local_storage: Optional['LocalStorage'] = None, logger_fun: Optional[Callable[[str, Optional[str], Optional[str], bool], logging.Logger]] = None, *, initialize: bool = True):
+    def __init__(self, operation_id: str = None, run_id: str = None, app_id: str = None, app_cfg: Dict[str, Any] = None, app_secrets: Dict[str, str] = None, msg_url: str = None, email: Optional[str] = None, dag_host_port: str = None, dag_host_port_extra: str = None, secret: str = None, dag_host_auth: str = None, single_pod: bool = None, indexes: int = None, index: Optional[int] = None, login: str = None, japp_logs_buffer: io.StringIO = None, local_storage: Optional['LocalStorage'] = None, logger_fun: Optional[Callable[[str, Optional[str], Optional[str], bool], logging.Logger]] = None, *, initialize: bool = True):
         if not initialize:  # need for types, use only copy with _fix_app_cfg
             return
         self.__operation_id = operation_id
         self.app_id = app_id
         self.run_id = run_id
         self.app_cfg: Union[MinimalCfg, Dict[str, Any]] = app_cfg
+        self.__app_secrets = app_secrets
         self.msg_url = msg_url
         self.email = email
         self.dag_key_value = Context._DagKeyValue(operation_id, run_id, dag_host_port_extra, secret, dag_host_auth, local_storage)
@@ -487,6 +489,8 @@ class Context(Generic[MinimalCfg]):
         self.__is_local = local_storage is not None
         self.__local_storage = local_storage
         self.__pauses = {}
+        self.journal = JournalProxy(CollectBuffer(C.JOURNAL_PATH(operation_id, run_id)))
+        self.state = StateProxy()
 
         if logger_fun is None:
             self.__logger_buf = io.StringIO()
@@ -769,7 +773,9 @@ class Context(Generic[MinimalCfg]):
 
     def get_scale_part(self, jdf: JDF) -> JDF:
         """return scale part of jdf"""
-        if isinstance(jdf, DFS) or isinstance(jdf, Sink):
+        if jdf is None:
+            return None
+        elif isinstance(jdf, DFS) or isinstance(jdf, Sink):
             return jdf._apply(self.get_scale_part)
         elif isinstance(jdf, Docs):
             div, mod = divmod(len(jdf), self.__indexes)
@@ -850,6 +856,13 @@ class Context(Generic[MinimalCfg]):
     def pause(self) -> Pause:
         return Pause(self.__pauses)
 
+    def secret(self, key: str, ignore_not_exist: bool = False) -> Optional[str]:
+        app_secret = self.__app_secrets.get(key)
+        if app_secret is None:
+            app_secret = self.app_cfg.get(key)
+        assert ignore_not_exist or app_secret is not None, f"not set secret {key}"
+        return app_secret
+
 
 def to_binary(smth: Any) -> bytes:
     return pickle.dumps(smth)
