@@ -46,7 +46,7 @@ class LocalRunner:
         self.__registry = JuliusRegistry(local_settings.import_dirs, logger_fun=self.__logger_fun, with_import_restrictions=local_settings.with_import_restrictions)
         self.__storage: LocalStorage = LocalStorage(local_settings, {k: scheme_class_columns(v) for k, v in self.__registry.schemes().items()}, self.__local_schemes)
         self.__registry._set_local_storage(self.__storage)
-        self.__local_settings = local_settings
+        self.local_settings = local_settings
         self.__secret = str(uuid.uuid4())   # FIXME one secret for operation_id
         self.__fail_dir = None
         self.__stream_event = Event()
@@ -73,7 +73,7 @@ class LocalRunner:
         assert state is None, f"pipeline already inited, operationId {init.operationId}"
 
         try:
-            state = State(init.operationId, init.schemesNames, logs_buffer=LocalLogsBuffer(self.__logger_fun, init.operationId))
+            state = State(init.operationId, self.__storage.schemes_names, logs_buffer=LocalLogsBuffer(self.__logger_fun, init.operationId))
             states[init.operationId] = state
             fail_storage = None if (path := self.__storage.fail_dir) is None else FailStorage(path, copy_obj=True, schemes=self.__local_schemes)
             j_pipeline = JuliusPipeline(init, self.__registry, state.logs_buffer, state.pauses, storage=self.__storage, fail_storage=fail_storage)
@@ -122,6 +122,8 @@ class LocalRunner:
             state.logs_buffer.write(f"{traceback.format_exc()}\n")
             raise Exception("run failed") from ex
         if not ok:
+            if (trace := state.pipeline.trace_error(run.runId)) is not None:
+                raise Exception(f"run failed: {trace}")
             raise Exception("run failed")
 
     async def __finish(self, metadata: FunMetadata):
@@ -162,16 +164,20 @@ class LocalRunner:
             profileMode=profile_mode,
             operationId=operation_id,
             dagHost="",     # TODO ok?
+            dagUrlExtra="",
+            dagHostAuthLogin="malevich",
+            dagHostAuthPassword="pass",
 
-            login=self.__local_settings.login,
+            login=self.local_settings.login,
             pipeline=pipeline,
-            schemesNames=[],
             image=Image(ref=""),
+            index=0,
             hash="",
             processorIds=set(pipeline.processors.keys()),
             secret=self.__secret,
             singlePod=True,
             continueAfterProcessor=False,
+            saveFails=True,
         )
         await self.__init(pipeline)
         return operation_id
@@ -187,6 +193,9 @@ class LocalRunner:
             profileMode=profile_mode,
             operationId=operation_id,
             dagHost="",
+            dagUrlExtra="",
+            dagHostAuthLogin="malevich",
+            dagHostAuthPassword="pass",
 
             runId=run_id,
             kafkaInitRun=None,
@@ -218,7 +227,7 @@ class LocalRunner:
             cfg = fix_cfg(cfg)
 
         operation_id = await self.prepare(pipeline, None, debug_mode, profile_mode)
-        run_id = ""
+        run_id = "run"
         await self.run(operation_id, run_id, cfg, debug_mode, profile_mode)
         await self.stop(operation_id, run_id)
         return operation_id
@@ -333,7 +342,7 @@ class LocalRunner:
 
         prefix_collections = os.path.join(prefix, FailStorage.collections_dir)
         prefix_objects = os.path.join(prefix, FailStorage.objects_dir)
-        real_prefix_objects = os.path.join(MOUNT_PATH_OBJ, self.__local_settings.login)
+        real_prefix_objects = os.path.join(MOUNT_PATH_OBJ, self.local_settings.login)
         if force_copy_objects:
             shutil.copytree(prefix_objects, real_prefix_objects, dirs_exist_ok=True)
         else:
@@ -367,6 +376,8 @@ class LocalRunner:
                 arg = arg[0]
             if isinstance(arg, str):
                 arguments[name] = AlternativeArgument(collectionId=coll_id(arg))
+            elif arg is None:
+                pass    # not set in base struct (DummyCollection)
             else:
                 group = []
                 for subarg in arg:
