@@ -1,5 +1,6 @@
 import logging
 import sys
+import traceback
 from typing import Optional
 import malevich_app.export.secondary.const as C
 
@@ -23,3 +24,13 @@ def base_logger_fun(operation_id: str, run_id: Optional[str] = None, bind_id: Op
     handler.setFormatter(formatter)
     logger.addHandler(handler)
     return logger
+
+
+def _log_uncaught_exceptions(exc_type, exc_value, exc_traceback):
+    if issubclass(exc_type, KeyboardInterrupt):
+        sys.__excepthook__(exc_type, exc_value, exc_traceback)
+        return
+    logger = logging.getLogger("unhandled")
+    logger.error("".join(traceback.format_exception(exc_type, exc_value, exc_traceback)))
+
+sys.excepthook = _log_uncaught_exceptions
